close all

layers = [imageInputLayer([90 90 1])    % specification of the input image size, 90 by 90 pixels in grayvalues
    
% first layer specifications (see also manuscript for explanation: 21 by 21
% pixels convolution matrix
convolution2dLayer([21,21],25)
batchNormalizationLayer
reluLayer
maxPooling2dLayer(2,'Stride',2)

% -----
% second layer specifications (see also manuscript for explanation: 11 by 11
% pixels convolution matrix
convolution2dLayer([11,11],32)
batchNormalizationLayer
reluLayer
maxPooling2dLayer(2,'Stride',2)

% -----
% third layer specifications (see also manuscript for explanation: 6 by 6
% pixels convolution matrix
convolution2dLayer([6,6],64)
batchNormalizationLayer
maxPooling2dLayer(2,'Stride',2)

% -----
fullyConnectedLayer(1)
regressionLayer];


% definition of the three categories; targetFolder specifies location of
% training data
categories = {'Croissant','Slipper','Shear'};
targetFolder = 'C:\Users\Stephan\Desktop\CNN\CNN training data\finalAdpt';

% stores all the images with their information, i.e. labels (categories)
% and inherited folder structures
imds = imageDatastore(fullfile(targetFolder, categories), 'LabelSource', 'foldernames');

% shuffle imds 10 times
for idx = 1:10
    imds = shuffle(imds);
end

% define 5% validation data and 95% training data randomly from all images
N = numel(imds.Files);
vdx = randperm(N,round(0.05*N));
vdx = sort(vdx);

validationData = [];
validationVals = [];

for idx = 1:numel(vdx)
    image = imread(imds.Files{vdx(idx)});
    validationData(:,:,1,idx) = image;
end


newLabelsVec = [];
validIdx = [];

% assign values to (manually) predefined classes: 0 to shear, 127 to
% croissnats and -127 to slippers
for idx = 1:numel(imds.Files)
    if ~ismember(idx,vdx)
        if strcmp(string(imds.Labels(idx)),'Slipper')
            newLabelsVec(end+1,:) = -127;
        elseif strcmp(string(imds.Labels(idx)),'Croissant')
            newLabelsVec(end+1,:) = 127;
        elseif strcmp(string(imds.Labels(idx)),'Shear')
            newLabelsVec(end+1,:) = 0;
        end
        validIdx(end+1) = idx;
    else
        if strcmp(string(imds.Labels(idx)),'Slipper')
            validationVals(end+1,:) = -127;
        elseif strcmp(string(imds.Labels(idx)),'Croissant')
            validationVals(end+1,:) = 127;
        elseif strcmp(string(imds.Labels(idx)),'Shear')
            validationVals(end+1,:) = 0;
        end
    end
end

validationData = uint8(validationData);
validationVals = int8(validationVals);

% exclude validation data from training data
tbl = table(imds.Files(validIdx),newLabelsVec);


% specify training options, e.g. learning rate, maximum number of
% epochs,...
options = trainingOptions('sgdm','MaxEpochs',10,'InitialLearnRate',0.001,'ValidationData',{validationData,validationVals},'ExecutionEnvironment','parallel','Plots','training-progress');
[net,tr] = trainNetwork(tbl,layers,options);

l1weights = net.Layers(2).Weights;
weights = mat2gray(l1weights);
l1weightsRes = imresize(weights,5);

% show first convolutional layer
figure
montage(l1weightsRes)
title('First convolutional layer weights')

